/*
 * file:        ADC.h
 * purpose:     class for analog digital conversion
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef ADC_H
#define ADC_H
#include "halConfig.h"

#ifdef HAVE_ADC
#include "Pin.h"

namespace srd {

class ADConverter
{
  struct Values {
    int16_t  v[8];
    int16_t  temp[NUM_OVERSAMPLING][8];
    int8_t   cOversampling;
    int8_t   activeChannel;
    int8_t   activeMuxes;
    int8_t   completed;
    };
  static volatile struct Values values;
  static void conversionComplete() __asm__(STR(ADC_INTERRUPT)) __attribute__((__signal__, __used__, __externally_visible__));
  static void calcValues();
  static void setActiveChannel(int8_t channel) {
    ADC_CHANNEL_REGISTER &= 0xE0;
    ADC_CHANNEL_REGISTER |= channel & 0x1F;
    };

public:
  enum class RefVoltage {
       AREF,
       AVCC,                                               // needs capacitor at AREF Pin
       Internal = 3                                        // needs capacitor at AREF Pin
       };
  enum class PreScale {
       PS_2 = 1,
       PS_4,
       PS_8,
       PS_16,
       PS_32,
       PS_64,
       PS_128
       };
  enum class AutoTrigger {
       FreeRunning,
       AnalogComparator,
       ExtInterrupt,
       Timer0Compare,
       Timer0Overflow,
       Timer1CompareB,
       Timer1Overflow,
       Timer1Capture
       };
  ADConverter(RefVoltage reference, PreScale prescale, int8_t leftAdjusted = 0);

  static void setAutoTriggerMode(AutoTrigger trigger) {
    ADC_TRIGGER_REGISTER &= ~(static_cast<uint8_t>(trigger) << 5);
    ADC_TRIGGER_REGISTER |=   static_cast<uint8_t>(trigger) << 5;
    };

  static void start() {
    ADC_START_REGISTER |= 1 << ADEN | 1 << ADSC;
    };

  static void stop()  {
    ADC_START_REGISTER &= ~(1 << ADEN);
    };

  static void reset();
  static void enableAutoTrigger(int8_t enable = 1) {
    if (enable) {
       ADC_SET_AUTOTRIGGER;
       }
    else {
       ADC_CLEAR_AUTOTRIGGER;
       }
    };
  static void    enableChannel(Pin pin, int8_t enable = 1);
  static void    selectNextChannel();
  static int16_t getValue(int8_t channel) { return channel < 8 ? values.v[channel] : 0; }
  static int8_t  conversionCompleted()    { return values.completed; }
  };
};
#endif                                                     // HAVE_ADC
#endif
