/*
 * file:        Buzzer.h
 * purpose:     class to make noise
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef BUZZER_H
#define BUZZER_H
#include "halConfig.h"
#include "HWTimer.h"

#ifdef HAVE_BUZZER
namespace srd {

enum class Tones {                                         // closest timer tops for real frequencies
     ToneH4 = 253,
     ToneC5 = 238,
     ToneD5 = 212,
     ToneE5 = 189,
     ToneF5 = 178,
     ToneG5 = 159,
     ToneA5 = 141,
     ToneH5 = 126,
     ToneC6 = 118,
     ToneD6 = 105,
     ToneE6 = 94,
     ToneF6 = 88,
     ToneG6 = 79,
     ToneA6 = 70,
     ToneH6 = 62,
     ToneC7 = 59,
     ToneD7 = 52,
     ToneE7 = 46,
     ToneF7 = 44,
     ToneG7 = 39,
     ToneA7 = 34
     };

template<typename TimerClass>
class Buzzer
{
public:
  Buzzer()
   : timer(TimerMode::CTC_OCR, PreScale::PS_64) {
    timer.enableChannel(Channel::B, ChannelMode::Toggle);
    };

  void doBeep(Tones tone, int16_t millisDuration) {
    timer.setCompareValue(Channel::B, static_cast<int16_t>(tone));
    this->duration = millisDuration;
    timer.start();
    };
  void countDown() {
    if (duration > 0) {
       if (!--duration) timer.stop();
       }
    };

private:
  TimerClass timer;
  int16_t duration;
  };

}
#endif                                                     // HAVE_BUZZER
#endif
