/*
 * file:        Encoder.h
 * purpose:     class for usage of turning encoders (works together with system tick interrupt)
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2003-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef ENCODER_H
#define ENCODER_H
#include "Pin.h"
#include "PushButton.h"

#ifdef HAVE_ENCODER
namespace srd {

class DefaultEncoderCounterHandler
{
public:
  virtual void handleCounter(int8_t delta) {
    };
  };


class Encoder
{
public:
  Encoder(const Pin& left, const Pin& right, const PushButton& bt)
   : left(left)
   , right(right)
   , bt(bt)
   , encLast(0)
   , encDelta(0) {
    this->left.toInput();
    this->right.toInput();
    handler = &defHandler;
    };
  void handle1Increment();
  void handle2Increment();
  void handle4Increment();

  PushButton* pushButton() { return &bt; };
  void setCounterHandler(DefaultEncoderCounterHandler* handler) { this->handler = handler; };
  void clearCounterHandler() { this->handler = &defHandler; };

private:
  Pin left;
  Pin right;
  PushButton bt;
  DefaultEncoderCounterHandler defHandler;
  DefaultEncoderCounterHandler* handler;
  int8_t encLast;
  int8_t encDelta;
  };
}
#endif                                                     // HAVE_ENCODER
#endif
