/*
 * file:        PushButton.C
 * purpose:     class for usage of pushbuttons with debouncing by interrupt
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2004-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#ifndef PUSHBUTTON_H
#define PUSHBUTTON_H
#include "Pin.h"

namespace srd {

class PushButton
{
public:
  PushButton(const Pin& pin, const int8_t activeHigh=1)
   : pin(pin)
   , buttonPressed(0)
   , activeHigh(activeHigh) {
    this->pin.toInput();
    }
  void checkPressed();

  int8_t pressedCount() const { return buttonPressed; }
  void   reset() { buttonPressed = 0; }

private:
  Pin pin;
  int8_t buttonPressed;
  const int8_t activeHigh;
  };

};
#endif
