/*
 * file:        UArt.h
 * purpose:     class for serial communication
 * project:     c++ framework for avr controllers
 * author:      Reinhard Mantey
 * copyright:   2014-2016 (c) Creative commons - by-nc-sa - some rights reserved
 *              see http://creativecommons.org/licenses/by-nc-sa/3.0/ for details
 */
#include "UArt.h"
#include <string.h>

#ifdef HAVE_SERIAL_COMM

using namespace srd;

ByteHandler::~ByteHandler() {
  }

uint8_t Serial::status;
ByteHandler* SerialReceiver::byteHandler;
const uint8_t* SerialSender::data2Send;
int16_t SerialSender::sendDataSize;
int16_t SerialSender::bytesSent;


Serial::Serial(BaudRate br, int8_t receive, int8_t send) {
  setSpeed(br);
  setMode();
  enableReceiver(receive);
  enableSender(send);
  }


void SerialReceiver::receiveByteDone() {
  status = UART_STATUS_REGISTER;
  if (byteHandler) byteHandler->byteReceived(UART_DATA_REGISTER);
  }


void SerialSender::sendByteDone() {
  if (data2Send && bytesSent < sendDataSize) {
     ++bytesSent;
     UART_DATA_REGISTER = *data2Send++;
     }
  else {
     enableSenderInterrupt(0);
     sendDataSize = bytesSent = 0;
     data2Send = 0;
     }
  }


void SerialSender::sendBinary(const uint8_t* p, int16_t size) {
  data2Send = p;
  sendDataSize = size;
  enableSenderInterrupt(1);
  }


void SerialSender::sendString(const uint8_t* s) {
  data2Send = s;
  sendDataSize = strlen((const char *)s);
  enableSenderInterrupt(1);
  }

#endif
